unit Year2kD;

{ A simple Year 2000 date component which set a breakpoint for any dates entered
	in a 2 digit format for conversion to either 19xx or 20xx.  A default
    cutoff year of 20 is provided in the Create routine.

    By: Dave Beseke - Strategic Software Designs, Inc.  daveb@strategicsoftware.com

    10/9/1998

    Use however you want - entirely free software.  Modify as you wish.
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Mask;

type
  TYear2kDate = class(TMaskEdit)
  private
    FCutoffYear:    SmallInt;
    procedure CMExit(var Message: TCMExit);   message CM_EXIT;
    { Private declarations }
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    procedure ValidateEdit;
  published
    property cutoffYear: SmallInt read FCutoffYear write FCutoffYear default 20;
    { Published declarations }
  end;

procedure Register;

implementation

constructor TYear2kDate.Create(AOwner: TComponent);
begin
    inherited Create(AOwner);
    editmask := '!99/99/9999;1;_';
    width := 74;
    cutoffYear := 20;
end;

procedure TYear2kDate.CMExit(var Message: TCMExit);
var
   yy, mm, dd:              Word;
   yystr:                   String;
   i:                       SmallInt;
   theString:               String;
begin
    if IsMasked and not (csDesigning in ComponentState) then
        ValidateEdit;

    if text <> '  /  /    ' then
    begin
	    // parse out the month, day and year

	    theString := Text;
    	i := Pos('/', theString);
	    mm := StrToInt(Copy(theString, 1, i - 1));
    	theString := Copy(theString, i + 1, 20);
	    i := Pos('/', theString);
    	dd := StrToInt(Copy(theString, 1, i - 1));
	    yystr := Trim(Copy(theString, i + 1, 20));
    	yy := StrToInt(yystr);

	    // if no year, force to 1900
    	// if 1 digit, assume 2000-2009
	    // if 2 digits, use cutoff to decide
    	// if 3 digits, goofy user so add 1000 to be equally odd!!!
	    // if 4, leave it alone

    	case length(yystr) of
        	0:  yy := 1900;
	        1:  yy := yy + 2000;
    	    2:  if yy < fCutoffYear then yy := yy + 2000 else yy := yy + 1900;
        	3:  yy := yy + 1000;
	    end;

    	Text := Format('%02.02d/%02.02d/%04.04d', [mm, dd, yy]);
    end;

    inherited;
end;

procedure TYear2kDate.ValidateEdit;
begin
    inherited;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TYear2kDate]);
end;

end.
